/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.db;

import org.databene.commons.ObjectNotFoundException;
import org.databene.commons.collection.OrderedNameMap;

/**
 * Parent for classes that can contain packages and tables.<br/><br/>
 * Created: 05.12.2010 17:38:07
 * @since 0.4
 * @author Volker Bergmann
 */
public abstract class DbsPackageHolder {

	OrderedNameMap<DbsPackage> packages;
	OrderedNameMap<DbsTable> tables;
	
	public DbsPackageHolder() {
		this.packages = OrderedNameMap.createCaseIgnorantMap();
		this.tables = OrderedNameMap.createCaseIgnorantMap();
	}

	public DbsPackage getPackage(String name) {
		return packages.get(name);
	}

	public void addPackage(DbsPackage pkg) {
		packages.put(pkg.getName(), pkg);
	}

	public void addTable(DbsTable table) {
		tables.put(table.getName(), table);
	}

	public DbsTable getTable(String name, boolean required, boolean recursive) {
		DbsTable result = tables.get(name);
		if (result != null)
			return result;
		for (DbsPackage pkg : packages.values()) {
			DbsTable table = pkg.getTable(name, false, true);
			if (table != null)
				return table;
		}
		if (required)
			throw new ObjectNotFoundException("Table '" + name + "' not found");
		return null;
	}

}
