/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.SystemInfo;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.query.DefectQueryCheck;
import org.w3c.dom.Element;

/**
 * Parses the &lt;unique&gt; part of a &lt;check&gt; and creates a SQL defect query.<br/><br/>
 * Created: 26.05.2011 12:28:24
 * @since 0.8
 * @author Volker Bergmann
 */
public class UniqueKeyParser extends DbSanityXMLParser { // TODO v0.8.2 rename to UniqueParser

	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet("columns");

	public UniqueKeyParser() {
		super("unique", REQUIRED_ATTRIBUTES, null, DefectQueryCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
	    DefectQueryCheck parent = (DefectQueryCheck) this.parent(parentPath);
		String table = parent.getTable();
		if (StringUtil.isEmpty(table))
			throw new SyntaxError("Missing 'table' spec in outer <check>", XMLUtil.format(element));

		// parse XML element
		String columns = getRequiredAttribute("columns", element);
		
		// create query
	    String LF = SystemInfo.getLineSeparator();
		String defectQuery = "select * from " + table + " where " + joinColumnValues(columns) + " in (" + LF +
			"	select " + joinColumnValues(columns) + " from " + table + 
			" group by " + columns + " having count(*) > 1" + LF +
			")";
		parent.setQuery(defectQuery);
	    return defectQuery;
	}

	private String joinColumnValues(String columns) {
		return columns.replace(",", " || '|' || ");
	}

}
